# This is vacuumms_types.py, defines the types 

import csv
import subprocess


## VACUUMMS types 


# superclass for all data objects

class VACUUMMS:

    def __init__(self, box):
        print("constructing VACUUMMS object")
        self.list = []
        self.box = box

    def dump(self):
        print("# box: " + str(self.box)) 

    # Placeholder in case we factor parts out 
    def to_file(self, filename):
        pass

    # write contents as dataset to hdf5 formatted file
    def to_hdf5(self, hdf5_filename, hdf5_path, hdf5_dataset_name):
        pass


class cav_record():
    def __init__(self, record):
        (self.x, self.y, self.z, self.d) = record
    def __str__(self):
        tab = "\t"
        return str(self.x) + tab + str(self.y) + tab + str(self.z) + tab + str(self.d)

class cav(VACUUMMS):

    def __init__(self, input_source, box):
        super().__init__(box)
        print("constructing VACUUMMS::cav object")

        # Capture the output stream instead of a file to create the cav object

        if (str(type(input_source)) == "<class '_io.BufferedReader'>"):
            for line in input_source:
                self.list.append(cav_record(line.decode().rstrip().split("\t")))

        else: # otherwise treat input_source as a file
            with open(input_file, "r", encoding="utf8") as cav_file:
                reader = csv.reader(cav_file, delimiter="\t")
                for row in reader:
                    self.list.append(cav_record(row))

    def dump(self):
        super().dump() # Dump PVT, box, etc
        for record in self.list:
            print(f"{record.x}\t{record.y}\t{record.z}\t{record.d}")

    # write contents to flat file (traditional)
#    def to_file(self, filename="/dev/null"):
    def to_file(self, filename=None):
        if (filename == None): filename=str(self) + ".cav"
        with open(filename, "w") as file:
            for record in self.list:
                print(f"{record.x}\t{record.y}\t{record.z}\t{record.d}", file=file)


class gfg_record():
    def __init__(self, record):
        (self.x, self.y, self.z, self.sigma, self.epsilon) = record

class gfg(VACUUMMS):
    def __init__(self, input_file, box):
        super().__init__(box)
        print("constructing VACUUMMS::gfg object from "+input_file + " with box = " + str(box))
        self.gfg_list = []

        with open(input_file, "r", encoding="utf8") as gfg_file:
            reader = csv.reader(gfg_file, delimiter="\t")
            for row in reader:
                self.gfg_list.append(gfg_record(row))

    def dump(self):
        super().dump() # Dump PVT, box, etc
        for record in self.gfg_list:
            print(f"{record.x}\t{record.y}\t{record.z}\t{record.sigma}\t{record.epsilon}")


class cls_record():
    def __init__(self, record):
        (self.cluster, self.x, self.y, self.z, self.d) = record
    def __str__(self):
        tab = "\t"
        return str(self.cluster) + tab + str(self.x) + tab + str(self.y) + tab + str(self.z) + tab + str(self.d)

class cls(VACUUMMS):

    def __init__(self, input_source, box):
        super().__init__(box)
        print("constructing VACUUMMS::cls object")

        # Capture the output stream instead of a file to create the cls object

        if (str(type(input_source)) == "<class '_io.BufferedReader'>"):
            for line in input_source:
                self.list.append(cls_record(line.decode().rstrip().split("\t")))

        else: # otherwise treat input_source as a file
            with open(input_file, "r", encoding="utf8") as cls_file:
                reader = csv.reader(cls_file, delimiter="\t")
                for row in reader:
                    self.list.append(cls_record(row))

    def dump(self):
        super().dump() # Dump PVT, box, etc
        for record in self.list:
            print(f"{record.cluster}\t{record.x}\t{record.y}\t{record.z}\t{record.d}")

    # write contents to flat file (traditional)
    def to_file(self, filename=None):
        if (filename == None): filename=str(self) + ".cls"
        with open(filename, "w") as file:
            for record in self.list:
                print(f"{record.cluster}\t{record.x}\t{record.y}\t{record.z}\t{record.d}", file=file)


class fvi_record():
    def __init__(self, record):
        (self.x, self.y, self.z, self.energy) = record
    def __str__(self):
        tab = "\t"
        return str(self.x) + tab + str(self.y) + tab + str(self.z) + tab + str(self.energy)

class fvi(VACUUMMS):

    def __init__(self, input_source, box):
        super().__init__(box)
        print("constructing VACUUMMS::fvi object")

        # Capture the output stream instead of a file to create the fvi object

        if (str(type(input_source)) == "<class '_io.BufferedReader'>"):
            for line in input_source:
                self.list.append(fvi_record(line.decode().rstrip().split("\t")))

        else: # otherwise treat input_source as a file
            with open(input_file, "r", encoding="utf8") as fvi_file:
                reader = csv.reader(fvi_file, delimiter="\t")
                for row in reader:
                    self.list.append(fvi_record(row))

    def dump(self):
        super().dump() # Dump PVT, box, etc
        for record in self.list:
            print(f"{record.x}\t{record.y}\t{record.z}\t{record.energy}")

    # write contents to flat file (traditional)
    def to_file(self, filename=None):
        if (filename == None): filename=str(self) + ".fvi"
        with open(filename, "w") as file:
            for record in self.list:
                print(f"{record.x}\t{record.y}\t{record.z}\t{record.energy}", file=file)


# It's a degenerate case, since each record contains only one value, but treat as others
class dst_record():
    def __init__(self, record):
        (self.value) = record
    def __str__(self):
        return str(self.value)

class dst(VACUUMMS):

    def __init__(self, input_source, box):
        super().__init__(box)
        print("constructing VACUUMMS::dst object")

        # Capture the output stream instead of a file to create the dst object

        if (str(type(input_source)) == "<class '_io.BufferedReader'>"):
            for line in input_source:
                self.list.append(dst_record(line.decode().rstrip().split("\t")))

        else: # otherwise treat input_source as a file
            with open(input_file, "r", encoding="utf8") as dst_file:
                reader = csv.reader(dst_file, delimiter="\t")
                for row in reader:
                    self.list.append(dst_record(row))

    def dump(self):
        super().dump() # Dump PVT, box, etc
        for record in self.list:
            print(f"{record.value}")

    # write contents to flat file (traditional)
    def to_file(self, filename=None):
        if (filename == None): filename=str(self) + ".dst"
        with open(filename, "w") as file:
            for record in self.list:
                print(f"{record.value}", file=file)


class hst_record():
    def __init__(self, record):
        (self.value, self.frequency) = record
    def __str__(self):
        tab = "\t"
        return str(self.value) + tab + str(self.frequency)

class hst(VACUUMMS):

    def __init__(self, input_source, box):
        super().__init__(box)
        print("constructing VACUUMMS::hst object")

        # Capture the output stream instead of a file to create the hst object

        if (str(type(input_source)) == "<class '_io.BufferedReader'>"):
            for line in input_source:
                self.list.append(hst_record(line.decode().rstrip().split("\t")))

        else: # otherwise treat input_source as a file
            with open(input_file, "r", encoding="utf8") as hst_file:
                reader = csv.reader(hst_file, delimiter="\t")
                for row in reader:
                    self.list.append(hst_record(row))

    def dump(self):
        super().dump() # Dump PVT, box, etc
        for record in self.list:
            print(f"{record.value}\t{record.frequency}")

    # write contents to flat file (traditional)
    def to_file(self, filename=None):
        if (filename == None): filename=str(self) + ".hst"
        with open(filename, "w") as file:
            for record in self.list:
                print(f"{record.value}\t{record.frequency}", file=file)





# This is vacuumms_operations.py, defines the operations 

import csv
import subprocess

from vacuumms_types import *

## VACUUMMS operations


def ddx(gfg, # box=[6.0, 6.0, 6.0],
             seed=1,
             randomize=False,
             characteristic_length=1.0,
             characteristic_energy=1.0,
             precision_parameter=0.001,
             n_steps=1000, 
             show_steps=False,
             verlet_cutoff=100.0,
             n=1,
             volume_sampling=False,
             include_center_energy=False,
             min_diameter=0.0):

    ddx_arglist = ['ddx', '-box', str(gfg.box[0]), str(gfg.box[1]), str(gfg.box[2]) ]
    ddx_arglist.append("-n_steps")
    ddx_arglist.append(str(n_steps))
    ddx_arglist.append("-seed")
    ddx_arglist.append(str(seed))
    if (randomize): ddx_arglist.append("-randomize")
    ddx_arglist.append("-characteristic_length")
    ddx_arglist.append(str(characteristic_length))
    ddx_arglist.append("-characteristic_energy")
    ddx_arglist.append(str(characteristic_energy))
    ddx_arglist.append("-precision_parameter")
    ddx_arglist.append(str(precision_parameter))
    ddx_arglist.append("-n_steps")
    ddx_arglist.append(str(n_steps))
    if (show_steps): ddx_arglist.append("-show_steps")
    ddx_arglist.append("-verlet_cutoff")
    ddx_arglist.append(str(verlet_cutoff))
    ddx_arglist.append("-n")
    ddx_arglist.append(str(n))
    if (volume_sampling): ddx_arglist.append("-volume_sampling")
    if (include_center_energy): ddx_arglist.append("-include_center_energy")
    ddx_arglist.append("-min_diameter")
    ddx_arglist.append(str(min_diameter))

    tab = str("\t")
    newline = str("\n")

    # open the process for ddx, and write gfg info
    with subprocess.Popen(ddx_arglist, stdin=subprocess.PIPE, stdout=subprocess.PIPE) as process:
        print("FTW subprocess for ddx: " + str(process))
        for row in gfg.gfg_list:
            line = str(row.x)+tab+str(row.y)+tab+str(row.z)+tab+str(row.sigma)+tab+str(row.epsilon)+newline
            process.stdin.write(bytes(line, encoding='utf-8'))

        # Done writing input data so close stdin
        process.stdin.close()

        # Capture the output stream to create cav object
        ddx_retval = cav(process.stdout, gfg.box)

        process.stdout.close()

    return ddx_retval




#!/usr/bin/env python

import random as rnd
import subprocess

# create a dst file
f=open("x.dst", "w")
for i in range(256):
    f.write(str(rnd.randrange(256))+'\n')
f.flush()
f.close()

# open the file in dst2hst. Can fix stderr later. 
f=open('x.dst', "r")
d2h=subprocess.Popen("dst2hst", stdout=subprocess.PIPE, stdin=f)
for line in d2h.stdout:
    print(line.rstrip().decode('ASCII'))

    



#!/usr/bin/env python

from setuptools import setup

## get long description from README
#with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
#    long_description = f.read()

setup(
    name='vacuumms',
    version='1.2.1',  
    description='A module for working with vacuumms',
    long_description='A module for working with vacuumms',
    url='http://www.vacuumms.org',
    author='Frank T. Willmore',
    author_email='frankwillmore@gmail.com',
    license='MIT License',
    classifiers=[
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Topic :: Scientific/Engineering',
#        'Operating System :: MacOS :: MacOS X',
#        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux'],
    install_requires=['numpy', 'scipy'],
#    extras_require={
#        "docs": ["sphinx", "sphinx_rtd_theme", "numpydoc"]
#    },
#    tests_require=["pytest"],
    packages=[
         'vacuumms',
#        'poo',
#        'poo.poo',
         ],
    include_package_data=True,
)
